@@ -60,6 +60,7 @@
 #include <asm/xmon.h>
 #include <asm/cputhreads.h>
 #include <mm/mmu_decl.h>
+#include <asm/archrandom.h>
 #include <asm/fadump.h>
 #include <asm/udbg.h>
 #include <asm/hugetlb.h>
@@ -176,17 +177,14 @@ EXPORT_SYMBOL_GPL(machine_power_off);
 void (*pm_power_off)(void);
 EXPORT_SYMBOL_GPL(pm_power_off);
 
-#ifdef CONFIG_ARCH_RANDOM
-bool __must_check arch_get_random_seed_long(unsigned long *v)
+size_t __must_check arch_get_random_seed_longs(unsigned long *v, size_t max_longs)
 {
 	if (ppc_md.get_random_seed)
 		return ppc_md.get_random_seed(v);
 
 	return false;
 }
-EXPORT_SYMBOL(arch_get_random_seed_long);
-
-#endif
+EXPORT_SYMBOL(arch_get_random_seed_longs);
 
 void machine_halt(void)
 {
